using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using Webolize.TimeTracker.Domain;

using Cuyahoga.Core.Util;
using Cuyahoga.Web.UI;
using Cuyahoga.Web.Util;

namespace Webolize.TimeTracker.Web.Admin
{
	/// <summary>
	/// Summary description for Edit Forums.
	/// </summary>
	public class Admin : ModuleAdminBasePage
	{
        protected System.Web.UI.WebControls.Repeater wf_BillableNotes;
        protected System.Web.UI.WebControls.Repeater wf_BillableTypes;
        protected System.Web.UI.WebControls.Repeater wf_Multipliers;
        protected System.Web.UI.WebControls.Repeater wf_ServiceTypes;
        protected System.Web.UI.WebControls.Repeater wf_ExpenseCodes;
        protected System.Web.UI.WebControls.LinkButton wf_NewBillableNote;
        protected System.Web.UI.WebControls.LinkButton wf_NewBillableType;
        protected System.Web.UI.WebControls.LinkButton wf_NewMultiplier;
		protected System.Web.UI.WebControls.LinkButton wf_NewServiceType;
		protected System.Web.UI.WebControls.LinkButton wf_NewExpenseCode;
		protected System.Web.UI.WebControls.TextBox wf_LicenseCode;
        protected System.Web.UI.WebControls.Literal wf_LicenseFile;
        protected System.Web.UI.WebControls.Label wf_IsLicensed;
        protected System.Web.UI.WebControls.Literal wf_LicenseType;
        protected System.Web.UI.WebControls.HyperLink wf_LicensedToCompany;
        protected System.Web.UI.WebControls.Literal wf_ReleaseDate;
        protected System.Web.UI.WebControls.Literal wf_LicensedUntil;

		private TimeTrackerModule _module;

		private void Page_Load(object sender, System.EventArgs e)
		{
			base.Title = "Webolize TimeTracker Administration";
			this._module = base.Module as TimeTrackerModule;

			if (! this.IsPostBack)
			{
                BindData();
			}
		}

        private void BindData()
        {
            TimeTrackerDB ttdb = new TimeTrackerDB( _module.SessMgr );

            // Service Types
            wf_ServiceTypes.DataSource = ttdb.GetAllServiceTypes();
            wf_ServiceTypes.DataBind();

            // Expense Codes
            wf_ExpenseCodes.DataSource = ttdb.GetAllExpenseCodes();
            wf_ExpenseCodes.DataBind();

            // Billable Multipliers (tax)
            wf_Multipliers.DataSource = ttdb.GetAllBillableMultipliers();
            wf_Multipliers.DataBind();

            // Billable Item Types
            wf_BillableTypes.DataSource = ttdb.GetAllBillableItemTypes();
            wf_BillableTypes.DataBind();

            // Billable Notes
            wf_BillableNotes.DataSource = ttdb.GetAllBillableNotes();
            wf_BillableNotes.DataBind();

            // Licensing
            wf_IsLicensed.Text = _module.IsLicensed.ToString();
            if ( _module.IsLicensed ) {
                wf_IsLicensed.ForeColor = System.Drawing.Color.Green;
                wf_LicensedToCompany.ForeColor = System.Drawing.Color.Black;
            } else {
                wf_IsLicensed.ForeColor = System.Drawing.Color.Red;
                wf_LicensedToCompany.NavigateUrl = UrlHelper.GetUrlFromSection( this._module.Section ) + "/View/Settings";
            }

            wf_LicenseFile.Text = _module.LicenseFile;
            wf_LicenseType.Text = _module.LicenseType;
            wf_LicensedToCompany.Text = _module.CompanyName;
            wf_ReleaseDate.Text = _module.ReleaseDate.ToShortDateString();
            wf_LicensedUntil.Text = "(You can use any version released before:)<br /> " + _module.LicensedUntil.ToShortDateString();
            if ( _module.LicensedUntil == DateTime.MinValue ) {
                wf_LicensedUntil.Visible = false;
            }
        }

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.wf_ServiceTypes.ItemDataBound += new System.Web.UI.WebControls.RepeaterItemEventHandler(this.rptForumCategories_ItemDataBound);
            this.wf_ExpenseCodes.ItemDataBound += new System.Web.UI.WebControls.RepeaterItemEventHandler( this.wf_ExpenseCodes_ItemDataBound );
            this.wf_Multipliers.ItemDataBound += new System.Web.UI.WebControls.RepeaterItemEventHandler( this.wf_Multipliers_ItemDataBound);
            this.wf_BillableTypes.ItemDataBound += new System.Web.UI.WebControls.RepeaterItemEventHandler( this.wf_BillableTypes_ItemDataBound );
            this.wf_BillableNotes.ItemDataBound += new System.Web.UI.WebControls.RepeaterItemEventHandler( this.wf_BillableNotes_ItemDataBound );
			this.wf_NewServiceType.Click += new System.EventHandler(this.wf_NewServiceType_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void rptForumCategories_ItemDataBound(object sender, System.Web.UI.WebControls.RepeaterItemEventArgs e)
		{
            TTServiceType ttst = e.Item.DataItem as TTServiceType;

			HyperLink hplEdit = e.Item.FindControl("hpledit") as HyperLink;
			if (hplEdit != null)
			{
                hplEdit.NavigateUrl = String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditServiceType.aspx?NodeId={0}&SectionId={1}&ServiceTypeID={2}", this.Node.Id, this.Section.Id, ttst.ID.ToString() );
			}
		}

		private void wf_ExpenseCodes_ItemDataBound(object sender, System.Web.UI.WebControls.RepeaterItemEventArgs e)
		{
            TTExpenseCode ttec = e.Item.DataItem as TTExpenseCode;

			HyperLink hplEdit = e.Item.FindControl("hpledit") as HyperLink;
			if (hplEdit != null)
			{
                hplEdit.NavigateUrl = String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditExpenseCode.aspx?NodeId={0}&SectionId={1}&ExpenseCodeID={2}", this.Node.Id, this.Section.Id, ttec.ID.ToString() );
			}
		}

        private void wf_Multipliers_ItemDataBound( object sender, System.Web.UI.WebControls.RepeaterItemEventArgs e )
		{
            TTBillableMultiplier ttbm = e.Item.DataItem as TTBillableMultiplier;

            HyperLink wf_EditMultiplier = e.Item.FindControl( "wf_EditMultiplier" ) as HyperLink;
            if ( wf_EditMultiplier != null )
			{
                wf_EditMultiplier.NavigateUrl = String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditMultiplier.aspx?NodeId={0}&SectionId={1}&MultiplierID={2}", this.Node.Id, this.Section.Id, ttbm.ID.ToString() );
			}
		}

        private void wf_BillableTypes_ItemDataBound( object sender, System.Web.UI.WebControls.RepeaterItemEventArgs e )
		{
            TTBillableType ttbt = e.Item.DataItem as TTBillableType;

            HyperLink wf_EditMultiplier = e.Item.FindControl( "wf_EditBillableType" ) as HyperLink;
            if ( wf_EditMultiplier != null )
			{
                wf_EditMultiplier.NavigateUrl = String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditBillableType.aspx?NodeId={0}&SectionId={1}&BillableTypeID={2}", this.Node.Id, this.Section.Id, ttbt.ID.ToString() );
			}
		}

        private void wf_BillableNotes_ItemDataBound( object sender, System.Web.UI.WebControls.RepeaterItemEventArgs e )
		{
            TTBillableNote ttbn = e.Item.DataItem as TTBillableNote;

            HyperLink wf_EditNote = e.Item.FindControl( "wf_EditBillableNote" ) as HyperLink;
            if ( wf_EditNote != null )
			{
                wf_EditNote.NavigateUrl = String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditBillableNote.aspx?NodeId={0}&SectionId={1}&BillableNoteID={2}", this.Node.Id, this.Section.Id, ttbn.ID.ToString() );
			}
		}

		private void wf_NewServiceType_Click(object sender, System.EventArgs e)
		{
            Response.Redirect( String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditServiceType.aspx?NodeId={0}&SectionId={1}&ServiceTypeId=-1", this.Node.Id, this.Section.Id ) );
		}

        protected void wf_NewExpenseCode_Click( object sender, EventArgs e )
        {
            Response.Redirect( String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditExpenseCode.aspx?NodeId={0}&SectionId={1}&ExpenseCodeId=-1", this.Node.Id, this.Section.Id ) );
        }

        protected void wf_NewMultiplier_Click( object sender, EventArgs e )
        {
            Response.Redirect( String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditMultiplier.aspx?NodeId={0}&SectionId={1}&MultiplierId=-1", this.Node.Id, this.Section.Id ) );

        }

        protected void wf_NewBillableType_Click( object sender, EventArgs e )
        {
            Response.Redirect( String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditBillableType.aspx?NodeId={0}&SectionId={1}&BillableTypeId=-1", this.Node.Id, this.Section.Id ) );

        }

        protected void wf_NewBillableNote_Click( object sender, EventArgs e )
        {
            Response.Redirect( String.Format( "~/Modules/Webolize-TimeTracker/Admin/EditBillableNote.aspx?NodeId={0}&SectionId={1}&BillableNoteId=-1", this.Node.Id, this.Section.Id ) );

        }
	}
}
